from typing import Any, Dict
import os
from bs4_web_scraper.file_handler import FileHandler
import bs4_web_scraper
import warnings
from dotenv import load_dotenv, find_dotenv, dotenv_values
import string
import random

from .jcrypt import JSONCrypt


SECRET_KEY_ALLOWED_CHARS = string.ascii_letters + string.digits + string.punctuation

env_variables = [
    "DJSM_SECRETS_FILE_PATH",
    "DJSM_SECRET_KEY_NAME",
    "DJSM_SECRET_KEY_FILE_PATH",
]


def find_and_load_env_var():
    """Load environment variables from .env file"""
    try:
        load_dotenv(find_dotenv('.env', raise_error_if_not_found=True), override=True)
    except Exception:
        raise EnvLoadError("Could not load environmental variables because '.env' file was not found. Create one!")


def check_setup_ok():
    """Check that an .env file is present and has been properly setup"""
    setup_ok = True
    print("DJSM: Ensuring setup is OK\n")
    try:
        print("DJSM: Searching for .env file...\n")
        find_and_load_env_var()
    except EnvLoadError:
        setup_ok = False
        print("DJSM: Could not load .env file or .env file not found!\n")
        return setup_ok

    print("DJSM: Checking that .env file has been properly setup...\n")
    env_file_dict = dotenv_values(find_dotenv(raise_error_if_not_found=True))
    if not env_file_dict.get('DJSM_SECRETS_FILE_PATH', None):
        print('DJSM: DJSM_SECRETS_FILE_PATH not set in .env file\n')
        setup_ok = False
    if setup_ok:
        print('DJSM: Setup OK!\n')
    return setup_ok
 


class EnvLoadError(Exception):
    """Unable to load .env file mainly because it was not found"""


class CryptKeysNotFound(Exception):
    """Secret encryption or/and decryption keys not found"""


class DjangoJSONSecretManager:
    """
    ### Class that helps manage secrets in Django applications

    :param path_to_secret_file: path the file where secrets are stored or secrets will be stored.

    EXAMPLE:
    >>> djsm = DjangoJSONSecretManager('.secret_hidden_folder/secret_file.json')
    >>> # SECURITY WARNING: keep the secret key used in production secret!
        # generate secret key if it does not exist
    >>> SECRET_KEY = djsm.get_or_create_secret_key()
    """

    __django_secret_key_name: str = 'DJANGO_SECRET_KEY'

    def __init__(self, path_to_secret_file: str):
        if not path_to_secret_file.endswith('.json'):
            raise ValueError('Secret file must be a json file')

        self.__path_to_secret_file = path_to_secret_file
        self.__django_secret_key_file_path = os.getenv('DJSM_SECRET_KEY_FILE_PATH') if os.getenv('DJSM_SECRET_KEY_FILE_PATH') else self.__path_to_secret_file
        self.__django_secret_key_name = os.getenv('DJSM_SECRET_KEY_NAME') if os.getenv('DJSM_SECRET_KEY_NAME') else self.__django_secret_key_name


    @property
    def __path_to_key_file(self):
        return f"{os.path.dirname(self.__path_to_secret_file)}\cryptkeys.json"
    

    def __setattr__(self, __name: str, __value: Any) -> None:
        if "path" in __name:
            __value = os.path.abspath(__value)
        return super().__setattr__(__name, __value)


    def encrypt(self, secret: Dict) -> Dict:
        """
        Encrypts a secret using the encryption key.

        :param secret: secret to be encrypted
        :return: encrypted secret
        """
        if not isinstance(secret, dict):
            raise TypeError("secret must be a dict")
        
        f_key, pub_key = None, None
        try:
            crypt_keys_file_hdl = FileHandler(self.__path_to_key_file)
            crypt_keys: Dict = crypt_keys_file_hdl.read_file()
            f_key = crypt_keys.get('DJSM_FERNET_KEY', None)
            pub_key = crypt_keys.get('DJSM_RSA_PUBLIC_KEY', None)
            crypt_keys_file_hdl.close_file()
        except Exception:
            pass
        priv_key = os.getenv('DJSM_RSA_PRIVATE_KEY')

        if all([f_key, pub_key, priv_key]) == False:
            f_key, pub_key, priv_key = JSONCrypt.generate_key_as_str()
            crypt_keys = {
                'DJSM_FERNET_KEY': f_key,
                'DJSM_RSA_PUBLIC_KEY': pub_key,
            }
            crypt_keys_file_hdl = FileHandler(self.__path_to_key_file)
            crypt_keys_file_hdl.write_to_file(crypt_keys)
            crypt_keys_file_hdl.close_file()
            self.__remove_rsa_priv_key_from_env()
            env_file_hdl = self.__get_env_hdl()
            env_file_hdl.write_to_file(f'DJSM_RSA_PRIVATE_KEY = "{priv_key}"\n', write_mode='a+')
            env_file_hdl.close_file()
            self.reload_env()
            
        jcrypt = JSONCrypt.from_str(f_key, pub_key, priv_key)
        encrypted_secret = jcrypt.j_encrypt(secret)
        return encrypted_secret


    def decrypt(self, encrypted_secret: Dict) -> Dict:
        """
        Decrypts an encrypted secret using the encryption key.

        :param encrypted_secret: encrypted secret to be decrypted
        :return: decrypted secret
        """
        if not isinstance(encrypted_secret, dict):
            raise TypeError("encrypted_secret must be a dict")

        f_key, pub_key = None, None
        try:
            crypt_keys_file_hdl = FileHandler(self.__path_to_key_file)
            crypt_keys: Dict = crypt_keys_file_hdl.read_file()
            f_key = crypt_keys.get('DJSM_FERNET_KEY', None)
            pub_key = crypt_keys.get('DJSM_RSA_PUBLIC_KEY', None)
            crypt_keys_file_hdl.close_file()
        except Exception:
            pass
        priv_key = os.getenv('DJSM_RSA_PRIVATE_KEY')

        if all([f_key, pub_key, priv_key]) == False:
            raise CryptKeysNotFound("Crypt key(s) not found. Secrets will not be recoverable")

        jcrypt = JSONCrypt.from_str(f_key, pub_key, priv_key)
        decrypted_secret = jcrypt.j_decrypt(encrypted_secret)
        return decrypted_secret


    @staticmethod
    def __get_env_hdl():
        """Gets and returns the .env file FileHandler object"""
        return FileHandler(find_dotenv(raise_error_if_not_found=True), allow_any=True) 

    
    @staticmethod
    def validate_secret_key(secret_key: str):
        """Check that Django secret key is valid."""
        if not isinstance(secret_key, str):
            return False
        if len(secret_key) < 32:
            return False
        for char in secret_key:
            if char not in SECRET_KEY_ALLOWED_CHARS:
                return False
        return True

    
    @staticmethod
    def generate_django_secret_key(length: int = 50):
        """
        Return a randomly generated key of not less than 32 characters
        
        :param length: length of secret key to be generated.
        """
        if length < 32:
            raise ValueError('Secret key length cannot be less than 32 characters')
        return ''.join(random.choice(SECRET_KEY_ALLOWED_CHARS) for _ in range(length))


    def __remove_rsa_priv_key_from_env(self):
        """Removes rsa private key from .env file if present"""
        env_file_handler = self.__get_env_hdl()
        env_file_dict = dotenv_values(find_dotenv(raise_error_if_not_found=True))
        env_file_dict.pop('DJSM_RSA_PRIVATE_KEY', None)

        # Clear existing variables in .env file and re-write
        env_file_handler.clear_file()
        for key, value in env_file_dict.items():
            line = f'{key} = "{value}"\n'
            env_file_handler.write_to_file(line, write_mode='a+')
        return env_file_handler.close_file()


    def change_crypt_keys(self):
        """Change the encryption keys"""
        # Get existing secrets
        secret_key = self.load_secrets(self.__django_secret_key_file_path, decrypt=True) if self.__django_secret_key_file_path else {}
        secrets = self.load_secrets(self.__path_to_secret_file, decrypt=True)

        # Delete fernet and rsa public keys in JSON file - Just delete the cryptkeys.json file
        FileHandler(self.__path_to_key_file).delete_file()
        # Delete crypt key in .env file
        self.__remove_rsa_priv_key_from_env()

        # Re-write secrets while encrypting them with new keys
        if secret_key:
            self.write_secrets(secret_key, self.__django_secret_key_file_path, encrypt=True)
        self.write_secrets(secrets, self.__path_to_secret_file, encrypt=True)
        return None
        

    def get_or_create_secret_key(self, always_generate: bool = False):
        """
        Get Django secret key or create and saves a secret key in json secret file if it is non-existent.

        The secret key will be saved to the path specified in the constructor or as specified in the
        .env file

        :param always_generate: if True, a new secret key will be generated and saved
        even if one already exists in the secret file.
        :return: secret key
        """ 
        secrets = self.load_secrets(self.__django_secret_key_file_path, decrypt=True)
        if not secrets.get(self.__django_secret_key_name, None) or always_generate:
            print("DJSM: Generating Secret Key...\n")
            secrets[self.__django_secret_key_name] = self.generate_django_secret_key()
            self.write_secrets(secrets, self.__django_secret_key_file_path, overwrite=True, encrypt=True)
            print("DJSM: New Secret Key Generated Successfully\n")

        secret_key = self.get_secret(self.__django_secret_key_name)
        if not self.validate_secret_key(secret_key):
            warnings.warn("DJSM: Invalid Secret Key Found. Replacing Secret Key...")
            # Replace secret key if the secret key is not valid
            secret_key = self.change_secret_key()
        return secret_key


    def change_secret_key(self):
        """
        Replaces Django secret key with a new one.
        """
        self.get_or_create_secret_key(always_generate=True)
        return None


    def write_secrets(self, secrets: Dict[str, Any], path_to_file: str, overwrite: bool = False, encrypt: bool = False) -> None:
        """
        Writes the secrets in the given path.

        :param path_to_file: path to file in which secrets will be written.
        :param encrypt: whether to encrypt the secrets before writing in the file. 
        :return: None.
        """
        if not isinstance(secrets, dict):
            raise TypeError('Secret must be a dictionary')

        file_hdl = FileHandler(path_to_file)
        if not file_hdl.filetype == "json":
            raise TypeError("Secret file must be a json file")

        if secrets and encrypt:
            secrets = self.encrypt(secrets)
        if overwrite:
            file_hdl.write_to_file(secrets)
        else:
            file_hdl.update_json(secrets)
        return file_hdl.close_file()

    
    def load_secrets(self, path_to_file: str, decrypt: bool = False):
        """
        Loads the secrets from the given path.

        :param path_to_file: path to file containing secrets.
        :param decrypt: whether to decrypt the secrets in the file. 
        :return: a dictionary of secrets.
        """
        try:
            file_hdl = FileHandler(path_to_file)
            if not file_hdl.filetype == "json":
                raise TypeError("Secret file must be a json file")

            secrets = file_hdl.read_file()
            file_hdl.close_file()
            if not isinstance(secrets, dict):
                raise TypeError('Secrets must be a dictionary')
            if secrets and decrypt:
                secrets = self.decrypt(secrets)

        except bs4_web_scraper.FileError:
            secrets = {}
        return secrets


    def get_secret(self, key: str) -> Any | None:
        """
        Gets secret from json secret(s) file.

        :param key: key to get from the json file(s)
        :return: secret if found, None if not
        """ 
        secrets = self.load_secrets(self.__path_to_secret_file, decrypt=True)
        if self.__django_secret_key_file_path != self.__path_to_secret_file:
            try:
                secrets.update(self.load_secrets(self.__django_secret_key_file_path, decrypt=True))
            except:
                pass
        return secrets.get(key, None)


    def update_secrets(self, new_secrets: Dict[str, Any]) -> None:
        """
        Updates secrets in json secrets file.

        :param secret: secret to write to the json file
        :return: None
        """
        if not isinstance(new_secrets, dict):
            raise TypeError('Secret must be a dictionary')
        
        secrets = self.load_secrets(self.__path_to_secret_file, decrypt=True)
        _secrets = {**secrets, **new_secrets}
        return self.write_secrets(_secrets, self.__path_to_secret_file, encrypt=True)


    def clean_up(self):
        """
        Deletes all secrets in the json secrets, and secret key files. Removes all environment variables
        set by the package.

        This method is useful when you want to delete all secrets and start afresh.
        """
        self.write_secrets({}, self.__path_to_secret_file, overwrite=True, encrypt=False)
        if self.__django_secret_key_file_path != self.__path_to_secret_file:
            self.write_secrets({}, self.__django_secret_key_file_path, overwrite=True, encrypt=False)
        self.__remove_rsa_priv_key_from_env()

        for var in env_variables:
            os.environ.pop(var, None)
        return None


    def reload_env(self):
        """
        Reloads the environment variables from the .env file.
        """
        return find_and_load_env_var()


    def clean_up_and_reload(self):
        """
        Deletes all secrets in the json secrets, and secret key files. Removes all environment variables
        set by the package and reloads the environment variables from the .env file.

        This method is useful when you want to delete all secrets and start afresh.
        """
        self.clean_up()
        return self.reload_env()






from typing import Dict, List, Any
import warnings

from .crypt import Crypt


class JSONCrypt(Crypt):
    """
    ### A subclass of the Crypt class that encrypts and decrypts JSON objects.

    :param enc_fernet_key: encrypted fernet key string
    :param public_key: public key
    :param private_key: private key
    :param hash_algorithm: hash algorithm to use for signing and verifying.
    Supported algorithms are: 'SHA-1', 'SHA-224', 'SHA-256', 'SHA-384', 'SHA-512'.

    :attr rsa_key_strength: rsa encryption key strength.
    :attr sign_and_verify_key: whether to sign and verify the fernet key on encryption and decryption. Default to True.
    :attr suppress_warnings: whether to suppress all warnings during encryption and decryption.

    :prop rsa_key_length: rsa encryption key length.

    NOTE: The higher the encryption key strength, the longer it takes to encrypt and decrypt but the more secure it is.
    There a three levels
    """
    sign_and_verify_key = True
    suppress_warnings = False

    def j_encrypt(self, json_object: Any):
        """
        Encrypts a JSON object using the encryption key.

        :param json_object: JSON parsable object to be encrypted
        :return: encrypted JSON object
        """
        return getattr(self, f"encrypt_{type(json_object).__name__.lower()}")(json_object)

    
    def j_decrypt(self, json_object: Any):
        """
        Decrypts a JSON object using the encryption key.

        :param json_object: JSON parsable object to be decrypted
        :return: decrypted JSON object
        """
        return getattr(self, f"decrypt_{type(json_object).__name__.lower()}")(json_object)


    def encrypt_str(self, string: str) -> str:
        """
        Encrypts a string using the encryption key.

        :param string: string to be encrypted
        :return: encrypted string and signature
        """
        if not isinstance(string, str):
            raise TypeError("string must be a string")
        r = super().encrypt(string)
        return r


    def decrypt_str(self, cipher_string: str):
        """
        Decrypts an encrypted string using the encryption key.

        :param cipher_string: encrypted string to be decrypted
        :param signature: signature of the encrypted string
        :return: decrypted string as str, int, float or bool
        """
        if not isinstance(cipher_string, str):
            raise TypeError("encrypted_string must be a string")

        type_ = "str"
        if cipher_string.startswith(':ty-') and len(cipher_string.split('\u0000')) > 1:
            type_, cipher_string = cipher_string.split('\u0000')
        r = super().decrypt(cipher_string)
        if type_ == ":ty-ndbl:":
            return int(r)
        elif type_ == ":ty-dbl:":
            return float(r)
        elif type_ == ":ty-bln:":
            return bool(r)
        return r


    def encrypt_int(self, int_: int):
        """
        Encrypts an integer

        :param int_: integer to be encrypted
        :return: encrypted integer as a string
        """
        if not isinstance(int_, int):
            raise TypeError(int_)
        e_int_ = self.encrypt_str(str(int_))
        return f":ty-ndbl:\u0000{e_int_}"


    def encrypt_float(self, float_: float):
        """
        Encrypts a float

        :param float_: float to be encrypted
        :return: encrypted float as a string
        """
        if not isinstance(float_, float):
            raise TypeError(float_)
        e_float_ = self.encrypt_str(str(float_))
        return f":ty-dbl:\u0000{e_float_}"


    def encrypt_bool(self, bool_: bool):
        """
        Encrypts a boolean

        :param bool_: boolean to be encrypted
        :return: encrypted boolean as a string
        """
        if not isinstance(bool_, bool):
            raise TypeError(bool_)
        e_bool_ = self.encrypt_str(str(bool_))
        return f":ty-bln:\u0000{e_bool_}"


    def encrypt_tuple(self, tuple_: tuple):
        """
        Encrypts a tuple content

        :param tuple_: tuple containing contents to be encrypted
        :return: a list of encrypted content
        """
        if not self.suppress_warnings:
            warnings.warn("Tuples are not recommended for JSON", RuntimeWarning)
        if not isinstance(tuple_, tuple):
            raise TypeError(tuple_)
        e_tuple_ = self.encrypt_list(list(tuple_))
        return e_tuple_


    def encrypt_set(self, set_: set):
        """
        Encrypts a set content

        :param set_: set containing contents to be encrypted
        :return: a list of encrypted content
        """
        if not self.suppress_warnings:
            warnings.warn("Sets are not recommended for JSON", RuntimeWarning)
            print("set will be encrypted as a list")
        if not isinstance(set_, set):
            raise TypeError(set_)
        e_set_ = self.encrypt_list(list(set_))
        return e_set_


    def encrypt_list(self, list_: List):
        """
        Encrypts a list content

        :param secret: list containing contents to be encrypted
        :return: list of encrypted content
        """
        if not isinstance(list_, list):
            raise TypeError(list_)
        encrypted_list = []
        for item in list_:
            encrypted_item = getattr(self, f"encrypt_{type(item).__name__.lower()}")(item)
            encrypted_list.append(encrypted_item)
        return encrypted_list

    
    def decrypt_list(self, cipher_list: List):
        """
        Decrypts a list of encrypted content

        :param cipher_list: list of encrypted content to be decrypted
        :return: list of decrypted content
        """
        if not isinstance(cipher_list, list):
            raise TypeError(cipher_list)
        decrypted_list = []
        for item in cipher_list:
            decrypted_item = getattr(self, f"decrypt_{type(item).__name__.lower()}")(item)
            decrypted_list.append(decrypted_item)
        return decrypted_list

    
    def encrypt_dict(self, dict_: Dict):
        """
        Encrypts a dict content

        :param dict_: dictionary containing contents to be encrypted
        :return: dictionary of encrypted content
        """
        if not isinstance(dict_, dict):
            raise TypeError(dict_)
        encrypted_dict = {}
        for key, value in dict_.items():
            encrypted_value = getattr(self, f"encrypt_{type(value).__name__.lower()}")(value)
            encrypted_dict[key] = encrypted_value
        return encrypted_dict

    
    def decrypt_dict(self, cipher_dict: Dict):
        """
        Decrypts dict with encrypted content

        :param cipher_list: list of encrypted content to be decrypted
        :return: list of decrypted content
        """
        if not isinstance(cipher_dict, dict):
            raise TypeError(cipher_dict)
        decrypted_dict = {}
        for key, value in cipher_dict.items():
            decrypted_value = getattr(self, f"decrypt_{type(value).__name__.lower()}")(value)
            decrypted_dict[key] = decrypted_value
        return decrypted_dict        



import rsa
import base64
from cryptography.fernet import Fernet


SUPPOTED_RSA_KEY_LENGTHS = [1024, 2048, 4096]

ENCRYPTION_LEVELS = [
    (1, 1024),
    (2, 2048),
    (3, 4096)
]


class KeyVerificationError(Exception):
    """Fernet key is not verified. Might have been tampered with."""


class Crypt:
    """
    ### Encrypts and decrypts text using Fernet + RSA Encryption

    :param enc_fernet_key: encrypted fernet key string
    :param public_key: public key
    :param private_key: private key
    :param hash_algorithm: hash algorithm to use for signing and verifying.
    Supported algorithms are: 'SHA-1', 'SHA-224', 'SHA-256', 'SHA-384', 'SHA-512'.

    :attr rsa_key_strength: rsa encryption key strength
    :attr sign_and_verify_key: whether to sign and verify the fernet key on encryption and decryption.

    :prop rsa_key_length: rsa encryption key length

    NOTE: The higher the encryption key strength, the longer it takes to encrypt and decrypt but the more secure it is.
    There a three levels
    """
    rsa_key_strength = 1
    sign_and_verify_key = False
    hash_algorithm = 'SHA-512'

    def __init__(self, enc_fernet_key: bytes, rsa_public_key: rsa.PublicKey, rsa_private_key: rsa.PrivateKey, hash_algorithm: str = 'SHA-512'):
        self.enc_fernet_key = enc_fernet_key
        self.rsa_pub_key = rsa_public_key
        self.rsa_priv_key = rsa_private_key
        self.hash_algorithm = hash_algorithm

    @classmethod
    def _get_rsa_key_length(cls) -> int:
        return ENCRYPTION_LEVELS[cls.rsa_key_strength - 1][1]

    @property
    def rsa_key_length(self):
        return self._get_rsa_key_length()

    @classmethod
    def generate_key(cls):
        """
        Generates key tuple.

        :returns: a tuple of encrypted Fernet key, the rsa public key and rsa private key used to encrypt the Fernet key
        """
        f_key = Fernet.generate_key()
        nbits = cls._get_rsa_key_length()
        pub_key, priv_key = rsa.newkeys(nbits)
        enc_f_key = cls._encrypt_f_key(f_key, pub_key, priv_key)
        return enc_f_key, pub_key, priv_key

    @staticmethod
    def _rsa_key_to_str(rsa_key: rsa.PublicKey | rsa.PrivateKey, encoding: str = 'utf-8'):
        rsa_key_str = base64.urlsafe_b64encode(rsa_key.save_pkcs1()).decode(encoding=encoding)
        return rsa_key_str

    @staticmethod
    def _load_rsa_key_from_str(rsa_key_str: str, type_: str = "public", encoding: str = 'utf-8'):
        key_bytes = base64.urlsafe_b64decode(rsa_key_str.encode(encoding=encoding))
        if type_ == 'private':
            return rsa.PrivateKey.load_pkcs1(key_bytes)
        elif type_ == 'public':
            return rsa.PublicKey.load_pkcs1(key_bytes)
        raise ValueError('type_ must be either "private" or "public"')

    @staticmethod
    def _enc_f_key_to_str(enc_f_key_bytes: bytes, encoding: str = 'utf-8'):
        return base64.urlsafe_b64encode(enc_f_key_bytes).decode(encoding=encoding)

    @staticmethod
    def _load_enc_f_key_from_str(enc_f_key_str: str, encoding: str = 'utf-8'):
        return base64.urlsafe_b64decode(enc_f_key_str.encode(encoding=encoding))

    @classmethod
    def _encrypt_f_key(cls, f_key: bytes, rsa_pub_key: rsa.PublicKey, rsa_priv_key: rsa.PrivateKey = None):
        enc_f_key = rsa.encrypt(f_key, rsa_pub_key)
        if cls.sign_and_verify_key and rsa_priv_key:
            signature = cls._rsa_sign_fernet_key(f_key, rsa_priv_key)
            enc_f_key = b'\u0000'.join([enc_f_key, signature])
        return enc_f_key

    @classmethod
    def _decrypt_f_key(cls, enc_f_key: bytes, rsa_priv_key: rsa.PrivateKey, rsa_pub_key: rsa.PublicKey = None):
        if cls.sign_and_verify_key:
            enc_f_key, signature = enc_f_key.split(b'\u0000')
        dec_f_key = rsa.decrypt(enc_f_key, rsa_priv_key)
        if cls.sign_and_verify_key and rsa_pub_key:
            is_verified = cls._rsa_verify_fernet_key(dec_f_key, signature, rsa_pub_key)
            if not is_verified:
                raise KeyVerificationError
        return dec_f_key

    @classmethod
    def _rsa_sign_fernet_key(cls, fernet_key: bytes, rsa_priv_key: rsa.PrivateKey):
        """
        Signs the fernet key using the rsa private key

        :param fernet_key: fernet key to be signed
        :param rsa_priv_key: rsa private key
        :return: signature
        """
        signature = rsa.sign(fernet_key, rsa_priv_key, cls.hash_algorithm)
        return signature

    @classmethod
    def _rsa_verify_fernet_key(cls, fernet_key: bytes, signature: bytes, rsa_pub_key: rsa.PublicKey) -> bool:
        """
        Verifies a decrypted fernet key using the public key

        :param fernet_key: fernet key to be verified
        :param signature: signature to be verified
        :param rsa_pub_key: rsa public key
        """
        return rsa.verify(fernet_key, signature, rsa_pub_key) == cls.hash_algorithm

    @classmethod
    def generate_key_as_str(cls, encoding: str = 'utf-8'):
        """
        Generates a key tuple containing strings.
        
        :param encoding: encoding to be used to encode the key strings
        :return: encrypted Fernet key, rsa public key and rsa private key used to encrypted the Fernet key
        :rtype : str
        """
        enc_f_key, pub_key, priv_key = cls.generate_key()
        enc_f_key_str = cls._enc_f_key_to_str(enc_f_key, encoding=encoding)
        pub_key_str = cls._rsa_key_to_str(pub_key, encoding=encoding)
        priv_key_str = cls._rsa_key_to_str(priv_key, encoding=encoding)
        return enc_f_key_str, pub_key_str, priv_key_str

    @classmethod
    def from_str(cls, enc_fernet_key: str, rsa_public_key: str, rsa_private_key: str, encoding: str = 'utf-8'):
        """
        Creates an Crypt object from key strings

        :param enc_fernet_key: encrypted fernet key string
        :param public_key: public key string
        :param private_key: private key string
        :param encoding: encoding used to encode the key strings
        :return: Crypt object
        """
        enc_fernet_key = cls._load_enc_f_key_from_str(enc_fernet_key, encoding=encoding)
        pub_key: rsa.PublicKey = cls._load_rsa_key_from_str(rsa_public_key, 'public', encoding=encoding)
        priv_key: rsa.PrivateKey = cls._load_rsa_key_from_str(rsa_private_key, 'private', encoding=encoding)
        return cls(enc_fernet_key, pub_key, priv_key)

    def encrypt(self, string: str, encoding: str = 'utf-8'):
        """
        Encrypts a string using the fernet key

        :param string: string to be encrypted
        :param encoding: encoding to be used to decode and encode the string
        :return: encrypted string
        """
        if not isinstance(string, str):
            raise TypeError('string must be of type str')

        f_key = self._decrypt_f_key(self.enc_fernet_key, self.rsa_priv_key, self.rsa_pub_key)
        string_bytes = string.encode(encoding=encoding)
        cipher_bytes = Fernet(f_key).encrypt(string_bytes)
        cipher_string = cipher_bytes.decode(encoding=encoding)
        return cipher_string

    def decrypt(self, cipher_string: str, encoding: str = 'utf-8'):
        """
        Decrypts a string using the fernet key

        :param cipher_string: string to be decrypted
        :param encoding: encoding to be used to decode and encode the string
        :return: decrypted string
        """
        if not isinstance(cipher_string, str):
            raise TypeError('cipher_string must be of type str')

        f_key = self._decrypt_f_key(self.enc_fernet_key, self.rsa_priv_key, self.rsa_pub_key)
        cipher_bytes = cipher_string.encode(encoding=encoding)
        string_bytes = Fernet(f_key).decrypt(cipher_bytes)
        string = string_bytes.decode(encoding=encoding)
        return string



"""
----------------------------------
djsm - Django JSON Secrets Manager
==================================
djsm helps create, store, retrieve, update and manage secrets in Django

LICENSE: MIT
"""

__version__ = "0.0.1"
__author__ = "ti-oluwa"
__license__ = "MIT"

import os
import sys

from .manager import DjangoJSONSecretManager as DJSM
from .manager import (EnvLoadError, find_and_load_env_var, CryptKeysNotFound, check_setup_ok, env_variables)

__setup_ok = check_setup_ok()
if not __setup_ok:
    sys.exit()

# Pre-instantiate a DJSM object
djsm = DJSM(os.getenv('DJSM_SECRETS_FILE_PATH'))

